# =====================================================================
# $Id: swulib.awk 37 2007-05-02 14:41:02Z carlo $
#
# AWK functions for Shell-Witted Utilities.
#
# Copyright (c) 2001,2008 Carlo Strozzi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# =====================================================================

# =====================================================================
# string _strip(string s, [int mode])
#
# Strip leading and trailing blanks and TABs from the input string.
# Returns the stripped string. If 'mode' is set to '1', then multiple
# middle blanks are squeezed. If 'mode' is set to '2' then any
# newlines in 'string' are turned to blanks prior to stripping.
# =====================================================================

function _strip(s,m) {
  if (m == 2) gsub(/\n/," ",s)
  sub(/^[ \t]+/,_NULL,s)
  sub(/[ \t]+$/,_NULL,s)
  if (m) gsub(/  +/," ",s)
  return s
}

# =====================================================================
# string _b64decode(string value)
#
# Original code (C) March 10, 2003, Peter van Eerten
# Source: http://www.turtle.dds.nl/b64dec.awk
#
# Modified for SWU by Carlo Strozzi, November 10th, 2006
#
# Take a base64-encoded string and decode it to its original value. 
# Being mawk(1) not 8-bit clean, the original unencode string should
# not contain values that fall outside the LOCALE charset.
# =====================================================================

function _b64decode(value,	base1,base2,base3,base4,tmp,\
				byte1,byte2,byte3,byte4,result) {

  result = ""

  while (length(value) > 0) {
	# Specify byte values
	base1 = substr(value, 1, 1)
	base2 = substr(value, 2, 1)
	base3 = substr(value, 3, 1)
	base4 = substr(value, 4, 1)
	# Now find numerical position in _BASE64 string
	byte1 = index(_BASE64, base1) - 1
	if (byte1 < 0) byte1 = 0
	byte2 = index(_BASE64, base2) - 1
	if (byte2 < 0) byte2 = 0
	byte3 = index(_BASE64, base3) - 1
	if (byte3 < 0) byte3 = 0
	byte4 = index(_BASE64, base4) - 1
	if (byte4 < 0) byte4 = 0
	# Reconstruct ASCII string

	# gawk(1) compatibility test.
	if ((tmp=_lshift(_and(byte1, 63), 2) + _rshift(_and(byte2, 48), 4)))
				result = result sprintf("%c", tmp)

	# gawk(1) compatibility test.
	if ((tmp=_lshift(_and(byte2, 15), 4) + _rshift(_and(byte3, 60), 2)))
				result = result sprintf("%c", tmp)

	# gawk(1) compatibility test.
	if ((tmp=_lshift(_and(byte3, 3), 6) + byte4))
				result = result sprintf("%c", tmp)

	# Decrease incoming string by 4
	value = substr(value, 5)
  }
  return result
}

# =====================================================================
# string _b64encode(string value)
#
# Original code (C) March 6, 2003, Peter van Eerten
# Source: http://www.turtle.dds.nl/b64enc.awk
#
# Modified for SWU by Carlo Strozzi, November 10th, 2006
#
# Take a string and return its base64-encoded value. Being mawk(1) not
# 8-bit clean, the original string should not contain values that fall
# outside the LOCALE charset.
# =====================================================================

function _b64encode(value,	byte1,byte2,byte3,byte4,\
				base1,base2,base3,base4,result) {
  result = ""

  while (length(value) > 0) {
	# Specify byte values
	if (length(value) == 1){
		byte1 = _asc(substr(value, 1, 1))
		byte2 = 0
		byte3 = 0
	}
	if (length(value) == 2) {
		byte1 = _asc(substr(value, 1, 1))
		byte2 = _asc(substr(value, 2, 1))
		byte3 = 0
	}
	if (length(value) >= 3) {
		byte1 = _asc(substr(value, 1, 1))
		byte2 = _asc(substr(value, 2, 1))
		byte3 = _asc(substr(value, 3, 1))
	}
	# Create BASE64 values
	base1 = _rshift(byte1, 2)
	base2 = _lshift(_and(byte1, 3), 4) + _rshift(_and(byte2, 240), 4)
	base3 = _lshift(_and(byte2, 15), 2) + _rshift(_and(byte3, 192), 6)
	base4 = _and(byte3, 63)
	# Compose BASE64 string
	if (length(value) == 1) {
		result = result substr(_BASE64, base1 + 1, 1)
		result = result substr(_BASE64, base2 + 1, 1)
		result = result "=="
		value = ""
	}
	if (length(value) == 2) {
		result = result substr(_BASE64, base1 + 1, 1)
		result = result substr(_BASE64, base2 + 1, 1)
		result = result substr(_BASE64, base3 + 1, 1)
		result = result "="
		value = ""
	}
	if (length(value) >= 3) {
		result = result substr(_BASE64, base1 + 1, 1)
		result = result substr(_BASE64, base2 + 1, 1)
		result = result substr(_BASE64, base3 + 1, 1)
		result = result substr(_BASE64, base4 + 1, 1)
		value = substr(value, 4)
	}
  }
  return result
}

# =====================================================================
# string _and(string var, string x)
#
# Bitwise AND between 2 variables - var AND x
#
# Original code (C) March 6, 2003, Peter van Eerten
# Source: http://www.turtle.dds.nl/b64enc.awk
#
# Modified for SWU by Carlo Strozzi, November 10th, 2006
# =====================================================================

function _and(var,x,		l_res,l_i) {

  l_res=0;

  for (l_i=0; l_i < 8; l_i++){
        if (var%2 == 1 && x%2 == 1) l_res=l_res/2 + 128;
        else l_res/=2;
        var=int(var/2);
        x=int(x/2);
  }
  return l_res;
}

# =====================================================================
# string _lshift(string var, int x)
#
# Rotate bytevalue left x times
#
# Original code (C) March 6, 2003, Peter van Eerten
# Source: http://www.turtle.dds.nl/b64enc.awk
#
# Modified for SWU by Carlo Strozzi, November 10th, 2006
# =====================================================================

function _lshift(var,x) {

  while(x > 0){
    var*=2;
    x--;
  }
  return var;
}

# =====================================================================
# string _rshift(string var, int x)
#
# Rotate bytevalue right x times
#
# Original code (C) March 6, 2003, Peter van Eerten
# Source: http://www.turtle.dds.nl/b64enc.awk
#
# Modified for SWU by Carlo Strozzi, November 10th, 2006
# =====================================================================

function _rshift(var,x) {

  while(x > 0){
    var=int(var/2);
    x--;
  }
  return var;
}

# =====================================================================
# string _asc(char value)
#
# Return ASCII value of character. Being mawk(1) not 8-bit clean, the
# original value should not fall outside the LOCALE charset.
#
# Original code (C) March 6, 2003, Peter van Eerten
# Source: http://www.turtle.dds.nl/b64enc.awk
#
# Modified for SWU by Carlo Strozzi, November 10th, 2006
# =====================================================================

function _asc(char,		l_found,i) {
  for (i = 32; i < 127; i++){
      if (sprintf("%c", i) == char) l_found = i;
  }
  return l_found
}

